import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/interfaces/process/urm/urm_data_processor.dart';
import 'package:fis_measure/process/primitives/location.dart';

// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

import '../calculator.dart';

class URMLocationVelCal extends Calculator<Location, double> {
  URMLocationVelCal(Location ref) : super(ref);
  URMSimpleMeasureParams? param;
  @override
  void calculate() async {}

  @override
  Future<void> calculateAsync() async {
    if (ref.feature == null) return;
    try {
      if (ref.application is! URMApplication) {
        return;
      }
      handleURMSimpleMeasureParams();
      try {
        if (ref.feature == null) return;
        if (ref.application is! URMApplication) return;
        final URMApplication urmApplication = ref.application as URMApplication;

        URMMeasureProcessResult? outResult =
            await urmApplication.getURMMeasureResult(
          urmMeasureType: URMMeasureType.URMLocationVel,
          rOIType: URMROIType.placeHolder_0,
          srcDPoints: param!.srcDPoints,
        );

        if (outResult != null) {
          final feature = ref.feature!;

          for (var output in ref.meta.outputs) {
            if (output.name == MeasureTerms.URMLocationVel) {
              output.unit = VidUsUnit.mms;
              feature.updateFloatValue(
                  output, outResult.resultData, output.unit);
            }
          }
        } else {
          throw Exception("URM Measure API error");
        }
      } catch (e) {
        logger.e('URM Measure error: $e');
        return;
      }
      ref.application.updateRenderReady.emit(this, null);
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  /// 处理获取到的URM参数
  void handleURMSimpleMeasureParams() {
    if (ref.feature == null) return;
    try {
      if (ref.application is! URMApplication) {
        return;
      }
      Size urmResultSize = const Size(0, 0);
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final point = ref.feature!.point.clone();
      URMMeasureParams params =
          urmApplication.urmDataProcessor.getURMMeasureParams();

      param = URMSimpleMeasureParams(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.URMRect,
        srcDPoints: [
          UrmPoint(
            x: urmApplication.localToView(point).x * params.screenWidth,
            y: urmApplication.localToView(point).y * params.screenHeight,
          ),
        ],
      );
    } catch (e) {
      logger.e('URM handleURMSimpleMeasureParams error: $e');
    }
  }
}
