import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/view/mobile_view/widgets/icon_btn.dart';
import 'package:fis_measure/view/player/control_board/operate_bar.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class ImageParamsPanel extends StatefulWidget {
  const ImageParamsPanel({Key? key, this.ifShowPanel = false})
      : super(key: key);
  final bool ifShowPanel;

  @override
  _ImageParamsPanelState createState() => _ImageParamsPanelState();
}

class _ImageParamsPanelState extends State<ImageParamsPanel>
    with SingleTickerProviderStateMixin {
  late Animation<double> animation;
  late AnimationController controller;

  /// 抽屉组件的显示高度
  static const double _widgetHeight = 100.0;

  /// 组件边框高度
  static const double _borderWidth = 2.0;

  /// 组件的回弹空间
  static const double _bounceSpace = 40.0;

  /// 动画起始偏移量
  static const double _beginOffset =
      _widgetHeight + _bounceSpace + _borderWidth * 2;

  /// 动画结束偏移量
  static const double _endOffset = _bounceSpace + _borderWidth;

  @override
  initState() {
    super.initState();
    controller = AnimationController(
      duration: const Duration(milliseconds: 600),
      vsync: this,
    );
    //使用弹性曲线
    animation = CurvedAnimation(parent: controller, curve: Curves.elasticIn);
    animation = Tween(begin: _beginOffset, end: _endOffset).animate(animation)
      ..addListener(() {
        setState(() => {});
      });
  }

  @override
  dispose() {
    //路由销毁时需要释放动画资源
    controller.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ImageParamsPanel oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.ifShowPanel != widget.ifShowPanel) {
      if (widget.ifShowPanel) {
        animation =
            CurvedAnimation(parent: controller, curve: Curves.elasticOut);
        animation =
            Tween(begin: _beginOffset, end: _endOffset).animate(animation)
              ..addListener(() {
                setState(() => {});
              });
        controller.forward();
      } else {
        animation =
            CurvedAnimation(parent: controller, curve: Curves.elasticIn);
        animation =
            Tween(begin: _beginOffset, end: _endOffset).animate(animation)
              ..addListener(() {
                setState(() => {});
              });
        controller.reverse();
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    final playerController =
        Get.find<IPlayerController>() as VidPlayerController;
    return Transform(
      transform: Matrix4.translationValues(0, animation.value, 0),
      child: Container(
        padding: const EdgeInsets.only(bottom: _bounceSpace),
        decoration: BoxDecoration(
          color: Colors.black,
          borderRadius: const BorderRadius.only(
            topLeft: Radius.circular(10),
            topRight: Radius.circular(10),
          ),
          border: Border.all(color: Colors.grey, width: _borderWidth),
        ),
        child: SizedBox(
            width: 360,
            height: _widgetHeight,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                SingleIconButton(
                    icon: Icons.refresh, onPressed: playerController.resetTone),
                const OperateBar(
                  ifShowResetBtn: false,
                ),
              ],
            )),
      ),
    );
  }
}
