import 'package:fis_common/logger/logger.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/remedical.m.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/visuals/visual_area.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/process/language/measure_language.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_helper.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:fis_measure/view/measure/combo_widget.dart';
import 'package:fis_measure/view/mobile_view/mobile_right_panel/mobile_more_measure_item_dialog.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:vid/us/vid_us_unit.dart';
import 'package:vid/us/vid_us_visual_area_type.dart';

class MobileBaseMeasureItemBtn {
  ItemMeta itemMeta;
  String displayName;
  IconData icon;
  MobileBaseMeasureItemBtn(
      {required this.itemMeta, required this.displayName, required this.icon});
}

/// 移动端测量项选择器（继承自 测量项页面）
class MobileMeasureSelector extends FStatefulWidget {
  const MobileMeasureSelector({Key? key}) : super(key: key);

  @override
  FState<MobileMeasureSelector> createState() => _MobileMeasureSelector();
}

class MobileMoreMeasureItemModesModel {
  String modeName;
  String displayName;
  List<MeasureItemModel> availableItems;
  MobileMoreMeasureItemModesModel(
      {required this.modeName,
      required this.displayName,
      required this.availableItems});
}

class _MobileMeasureSelector extends FState<MobileMeasureSelector> {
  late final application = Get.find<IApplication>();

  /// 数据
  late final measureData = Get.find<MeasureDataController>();
  late final playerController = Get.find<IPlayerController>();

  /// 测量语言包
  final measureLanguage = MeasureLanguage();

  /// 当前选中的测量项目
  String activeName = "";

  /// 是否打开更多测量项列表
  bool showMore = false;

  /// 当前是否为组合测量项
  bool isCombo = false;

  /// 当前的子测量项下标
  int activeChildItemIndex = 0;

  /// 子测量项名称列表
  List<String> childItemLabels = [];

  /// 当前的组合测量项父类
  ITopMeasureItem? topMeasureItem;

  /// 更多测量项弹窗状态缓存
  MoreMeasureBackResult? moreMeasureBackResult;

  /// 写死的移动端基础测量项
  final List<MobileBaseMeasureItemBtn> _mobileBasicMeasureItemsList = [
    MobileBaseMeasureItemBtn(
      icon: Icons.linear_scale,
      displayName: i18nBook.measure.distance.t,
      itemMeta: ItemMeta(
        MeasureTypes.Distance,
        measureType: MeasureTypes.Distance,
        description: MeasureTypes.Distance,
        briefAnnotation: "D",
        outputs: [
          ItemOutputMeta(MeasureTypes.Distance, "Distance", VidUsUnit.cm),
        ],
      ),
    ),
    MobileBaseMeasureItemBtn(
      icon: Icons.border_style,
      displayName: i18nBook.measure.circumference.t,
      itemMeta: ItemMeta(
        MeasureTerms.Perimeter,
        measureType: MeasureTypes.AreaPerimeterTrace,
        description: MeasureTerms.Perimeter,
        briefAnnotation: "P",
        outputs: [
          ItemOutputMeta(MeasureTerms.Perimeter, "Perimeter", VidUsUnit.cm),
        ],
      ),
    ),
    MobileBaseMeasureItemBtn(
      icon: Icons.format_shapes,
      displayName: i18nBook.measure.area.t,
      itemMeta: ItemMeta(
        MeasureTerms.Area,
        measureType: MeasureTypes.AreaPerimeterTrace,
        description: MeasureTerms.Area,
        briefAnnotation: "A",
        outputs: [
          ItemOutputMeta(MeasureTerms.Area, "Area", VidUsUnit.cm2),
        ],
      ),
    ),
  ];

  /// 支持基础测量项的区域类型
  final List<VidUsVisualAreaType> _supportBasicMeasureAreaTypes = [
    VidUsVisualAreaType.Tissue,
    VidUsVisualAreaType.Flow,
  ];

  /// 是否已初始化完成所有测量项数据
  bool _isInitAllMeasureItems = false;

  /// 可用的模式 (测量项包含在内)
  List<MobileMoreMeasureItemModesModel> _availableModes = [];

  /// 当前的区域类型
  VidUsVisualAreaType? _currentAreaType = VidUsVisualAreaType.Tissue;

  /// 判断是否为特殊测量项
  bool get isSpecial => MeasurespecialsupportedTerms.items.contains(activeName);

  /// 是否允许使用基础测量项
  bool get _isEnableBaseMeasureItems =>
      _supportBasicMeasureAreaTypes.contains(_currentAreaType);

  @override
  FWidget build(BuildContext context) {
    return FSizedBox(
      width: 100,
      child: showMore
          ? _buildExtraMeasureItemPanel()
          : _buildBaseMeasureItemPanel(),
    );
  }

  /// 切换子测量项
  void changeChildItem(int index) {
    if (isCombo && topMeasureItem != null) {
      topMeasureItem!.switchChild(index);
    }
  }

  /// 切换测量项
  bool changeItem(ItemMeta itemMeta) {
    if (mounted) {
      setState(() {
        activeName = itemMeta.description;
      });
    }
    bool success = false;
    try {
      success = application.switchItem(itemMeta);
    } catch (e) {
      logger.e("changeItem failed", e);
    }
    return success;
  }

  @override
  void dispose() {
    super.dispose();
    application.activeMeasureItemChanged
        .removeListener(_onActiveMeasureItemChanged);
    application.visualAreaChanged.removeListener(_visualAreaChanged);
    final item = application.activeMeasureItem;
    if (item != null && item is ITopMeasureItem) {
      item.workingChildChanged.removeListener(_onWorkingChildChanged);
    }
  }

  @override
  void initState() {
    super.initState();
    _cancelCurrSelect();
    application.activeMeasureItemChanged
        .addListener(_onActiveMeasureItemChanged);
    application.visualAreaChanged.addListener(_visualAreaChanged);
    _currentAreaType = application.currentVisualArea.visualAreaType;
    _initMeasureItemsListAsync();
  }

  /// 构建基础测量项按钮
  FWidget _buildBaseMeasureBtn(
      ItemMeta itemMeta, bool ifActive, String displayName, IconData icon) {
    return FInkWell(
      onTap: () {
        HapticFeedback.mediumImpact();
        if (ifActive) {
          _cancelCurrSelect();
        } else {
          if (_isEnableBaseMeasureItems) {
            changeItem(itemMeta);
          } else {
            PromptBox.toast(i18nBook.measure.unsupportedMeasurementArea.t);
          }
        }
      },
      child: Container(
        margin: const EdgeInsets.fromLTRB(0, 10, 10, 10),
        padding: const EdgeInsets.fromLTRB(10, 0, 9, 0),
        decoration: BoxDecoration(
          border:
              Border.all(color: ifActive ? Colors.grey : Colors.transparent),
          borderRadius: BorderRadiusGeometry.lerp(
              BorderRadius.circular(10), BorderRadius.circular(10), 10),
        ),
        child: FColumn(
          children: [
            const FSizedBox(
              height: 10,
            ),
            FIcon(
              icon,
              color: ifActive ? Colors.white : Colors.grey,
            ),
            FText(
              displayName,
              style: TextStyle(
                color: ifActive ? Colors.white : Colors.grey,
              ),
            ),
            const FSizedBox(
              height: 10,
            ),
          ],
        ),
      ),
    );
  }

  /// 构建基础测量项面板
  FWidget _buildBaseMeasureItemPanel() {
    return FColumn(
      mainAxisAlignment: MainAxisAlignment.center,
      mainAxisSize: MainAxisSize.max,
      children: [
        ..._mobileBasicMeasureItemsList.map((e) {
          return _buildBaseMeasureBtn(
              e.itemMeta, e.itemMeta.name == activeName, e.displayName, e.icon);
        }).toList(),
        if (!application.isThirdPart) _buildChangeMoreMeasureItem(),
      ],
    );
  }

  /// 构建取消当前测量项按钮
  FWidget _buildCancelCurrMeasureItemBtn() {
    return FInkWell(
      onTap: () {
        _cancelCurrSelect();
        setState(() {
          showMore = false;
        });
        HapticFeedback.mediumImpact();
      },
      child: Container(
        margin: const EdgeInsets.fromLTRB(0, 10, 10, 10),
        padding: const EdgeInsets.fromLTRB(10, 0, 10, 0),
        decoration: BoxDecoration(
          border: Border.all(color: Colors.transparent),
          borderRadius: BorderRadiusGeometry.lerp(
              BorderRadius.circular(10), BorderRadius.circular(10), 10),
        ),
        child: FColumn(
          children: [
            const FIcon(
              Icons.cancel_outlined,
              color: Colors.grey,
            ),
            FText(
              i18nBook.common.cancel.t,
              style: const TextStyle(
                color: Colors.grey,
              ),
            ),
          ],
        ),
      ),
    );
  }

  /// 构建打开更多测量项按钮
  FWidget _buildChangeMoreMeasureItem() {
    return FInkWell(
      onTap: () async {
        HapticFeedback.mediumImpact();
        await _openMoreMeasureItemDialog();
      },
      child: Container(
        margin: const EdgeInsets.fromLTRB(0, 10, 10, 10),
        padding: const EdgeInsets.fromLTRB(10, 0, 10, 0),
        decoration: BoxDecoration(
          border: Border.all(color: Colors.transparent),
          borderRadius: BorderRadiusGeometry.lerp(
              BorderRadius.circular(10), BorderRadius.circular(10), 10),
        ),
        child: FColumn(
          children: [
            const FIcon(
              Icons.format_list_bulleted,
              color: Colors.grey,
            ),
            FText(
              i18nBook.measure.expandGroup.t,
              style: const TextStyle(
                color: Colors.grey,
              ),
            ),
          ],
        ),
      ),
    );
  }

  /// 构建子测量项按钮
  FWidget _buildChildItemBtn(int index, String displayName) {
    final bool isActive = index == activeChildItemIndex;
    return FInkWell(
      onTap: () {
        HapticFeedback.mediumImpact();
        changeChildItem(index);
      },
      child: FContainer(
        width: 500,
        height: 30,
        margin: const EdgeInsets.symmetric(vertical: 5),
        decoration: BoxDecoration(
            border: Border.all(color: isActive ? Colors.white : Colors.grey),
            borderRadius: BorderRadius.circular(5)),
        child: FCenter(
          child: FText(
            displayName,
            style: TextStyle(
              color: isActive ? Colors.white : Colors.grey,
            ),
          ),
        ),
      ),
    );
  }

  /// 构建子测量项选择器
  FWidget _buildChildItemBtns() {
    if (isSpecial) {
      //是否为特殊组合测量项
      return const SpecialItemHR();
    }
    if (!isCombo) return const FSizedBox();
    return FListView(
      children: List.generate(
        childItemLabels.length,
        (index) => _buildChildItemBtn(index, childItemLabels[index]),
      ),
    );
  }

  List<FWidget> _buildCurrentMeasureItem() {
    final measureLanguage = MeasureLanguage();
    final displayName = measureLanguage.t('measure', activeName);
    return [
      FText(
        i18nBook.measure.currentMeasureTool.t,
        textAlign: TextAlign.center,
        style: const TextStyle(color: Colors.grey, fontSize: 14),
      ),
      const FSizedBox(
        height: 10,
      ),
      FText(
        activeName,
        textAlign: TextAlign.center,
        style: const TextStyle(color: Colors.white),
      ),
      const FSizedBox(
        height: 5,
      ),
      if (!i18nBook.isCurrentEnglish)
        FText(
          displayName,
          textAlign: TextAlign.center,
          style: const TextStyle(color: Colors.grey, fontSize: 12),
        ),
    ];
  }

  /// 构建更多测量项面板
  FWidget _buildExtraMeasureItemPanel() {
    return FColumn(
      mainAxisAlignment: MainAxisAlignment.center,
      mainAxisSize: MainAxisSize.max,
      children: [
        const FSizedBox(
          height: 50,
        ),
        ..._buildCurrentMeasureItem(),
        const FSizedBox(
          height: 10,
        ),
        FExpanded(child: _buildChildItemBtns()),
        _buildCancelCurrMeasureItemBtn(),
        _buildChangeMoreMeasureItem(),
      ],
    );
  }

  /// 取消当前选中的测量项
  void _cancelCurrSelect() {
    changeItem(
      ItemMeta(
        "None",
        measureType: "None",
        description: "None",
        outputs: [
          ItemOutputMeta("None", "None", VidUsUnit.cm2),
        ],
      ),
    );
  }

  /// 过滤出当前支持的所有测量项
  List<ItemMetaDTO> _currSupportedItemFilter(MeasureModeDTO mode) {
    final supportedMeasureTypeName = [
      'TimeSpan', // StraightLine
      'Distance', // StraightLine
      "VerticalDistance", // StraightLine
      'AreaPerimeterTrace', // Trace
      "Depth", // Location
      "Velocity", // Location
      "MDepth", // Location
      "HipOneRay", // TwoRay
      "HipTwoRay", // ThreeRay
      "DepthToBaseLine", // DepthToBaseLine
      "AbRatioTwoVelocity", // TwoLocation
      "ResistivityIndexTwoLocationByEd", // TwoLocation
      "MaxPgTwoLocation", // TwoLocation
      "VolumeThreeDistance", // LWHStraightLine
      "ThreeDistanceMean", // LWHStraightLine
      "ThreeDistanceMax", // LWHStraightLine
      "AbRatioTwoArea", // TwoArea
      "StenosisTwoArea", // TwoArea
      "AreaStraightLine", // AreaStraightLine
      "AreaPerimeterSpline", // Spline
      "CurveLengthSpline", // Spline
      "AreaPerimeterPolyline", // Polyline
      "CurveLengthPolyline", // Polyline
      "PolyLineAngle", // PolyLineAngle
      "CardiacAxis", // TwolineAngle
      "Afi", // Afi
      "VolumeEllipse", // Ellipse
      "AreaPerimeterEllipse", // Ellipse
      "DopplerTrace", // MultiTrace
    ];
    List<ItemMetaDTO> _supportedItems = [];
    if (mode.availableGroups == null || mode.availableGroups!.isEmpty) {
      return _supportedItems;
    }
    for (MeasureGroupDTO group in mode.availableGroups!) {
      if (group.availableFolders == null || group.availableFolders!.isEmpty) {
        continue;
      }
      for (MeasureFolderDTO folder in group.availableFolders!) {
        if (folder.availableItems == null || folder.availableItems!.isEmpty) {
          continue;
        }
        for (ItemMetaDTO items in folder.availableItems!) {
          bool isSupported = false;
          if (supportedMeasureTypeName.contains(items.measureTypeName)) {
            isSupported = true;
          }
          if (items.multiMethodItems != null &&
              items.multiMethodItems!.isNotEmpty) {
            for (var item in items.multiMethodItems!) {
              if (supportedMeasureTypeName.contains(item.measureTypeName)) {
                if (item.isWorking) {
                  isSupported = true;
                }
              }
            }
          }

          if (items.methodChildItems != null &&
              items.methodChildItems!.isNotEmpty) {
            bool isAdd = true;
            for (var item in items.methodChildItems!) {
              if (!supportedMeasureTypeName.contains(item.measureTypeName)) {
                isAdd = false;
              }
            }
            if (isAdd) {
              isSupported = true;
            }
          }

          /// 排除不支持的项
          if (MeasureUnsupportedTerms.items.contains(items.name)) {
            isSupported = false;
          }

          // final cName = measureLanguage.t('measure', items.description ?? '');
          if (isSupported) {
            _supportedItems.add(items);

            // print("✅ 支持的测量项：${items.name} | ${items.measureTypeName} | $cName");
          } else {
            // print(
            //     "❌ 不支持的测量项：${items.name} | ${items.measureTypeName} | $cName");
          }
        }
      }
    }
    // print("${mode.modeName} 模式下过滤出 ${_supportedItems.length} 项");
    return _supportedItems;
  }

  /// 可能有些区域会重叠，无法手动区分，所以需要根据当前的区域来查找可以使用的测量项
  List<String> _findModesByCurrArea(IVisualArea currArea) {
    List<String> modes = [];
    List<IVisualArea> visualAreas = application.visuals[0].visualAreas;
    if (visualAreas.isEmpty) {
      return modes;
    }
    for (IVisualArea element in visualAreas) {
      if (element.displayRegion == currArea.displayRegion) {
        modes.add(element.mode.modeType.toString().split('.')[1]);
      }
    }
    return modes;
  }

  /// 获取当前帧信息来初始化当前测量项
  Future<List<MobileMoreMeasureItemModesModel>> _initMeasureItemsList() async {
    final vidImage = playerController.currentFrame;
    if (vidImage == null) return [];
    List<String> getModes = [];
    List<String> displayModeNames = [];
    for (var element in vidImage.visuals[0].modes) {
      getModes.add(element.type.toString().split('.')[1]);
      displayModeNames.add(element.displayName);
    }
    measureData.applicationModes = vidImage.visuals[0].modes;
    measureData.currentMode = vidImage.visuals[0].modes.first.type.name;
    var measureModeSelection = MeasureModeSelection(
      application.applicationName,
      application.categoryName,
      application.isThirdPart ? ['TPPTissue'] : getModes,
    );

    /// 请求数据
    final MeasureApplicationDTO? measureApplicationDTO = await MeasureDataHelper
        .getMeasureApplication
        .call(measureModeSelection);
    if (measureApplicationDTO == null ||
        measureApplicationDTO.availableModes == null) return [];

    /// 遍历 measureApplicationDTO.availableModes 拼装出新的 supportedModes
    List<MobileMoreMeasureItemModesModel> supportedModes = [];
    for (int i = 0; i < measureApplicationDTO.availableModes!.length; i++) {
      MeasureModeDTO mode = measureApplicationDTO.availableModes![i];
      List<ItemMetaDTO> supportedItems = _currSupportedItemFilter(mode);
      String displayName =
          displayModeNames.length > i ? displayModeNames[i] : '';
      MobileMoreMeasureItemModesModel modeModel =
          MobileMoreMeasureItemModesModel(
              modeName: mode.modeName ?? '',
              displayName: displayName,
              availableItems: _itemConverter(supportedItems));
      supportedModes.add(modeModel);
    }
    if (supportedModes.isNotEmpty) {
      _isInitAllMeasureItems = true;
    }
    return supportedModes;
  }

  void _initMeasureItemsListAsync() async {
    _availableModes = await _initMeasureItemsList();
  }

  List<MeasureItemModel> _itemConverter(List<ItemMetaDTO> items) {
    if (!i18nBook.isCurrentEnglish) {
      return items.map((e) {
        final cName = measureLanguage.t('measure', e.description ?? '');
        return MeasureItemModel(
          displayName: cName,
          itemMeta: e,
          category: e.categories?.first ?? '',
          searchKey: "${e.name?.toLowerCase()}$cName",
        );
      }).toList();
    } else {
      return items.map((e) {
        return MeasureItemModel(
          displayName: e.name ?? '',
          itemMeta: e,
          category: e.categories?.first ?? '',
          searchKey: "${e.name?.toLowerCase()}",
        );
      }).toList();
    }
  }

  /// 测量项变化事件监听
  /// 需要重新绑定子测量项
  void _onActiveMeasureItemChanged(sender, IMeasureItem? e) {
    if (e != null) {
      final item = application.activeMeasureItem!;
      if (item is ITopMeasureItem) {
        topMeasureItem = item;
        isCombo = true;
        item.switchChild(0);
        activeChildItemIndex = 0;
        item.workingChildChanged.addListener(_onWorkingChildChanged);
        childItemLabels = item.childItems.map((e) => (e.description)).toList();
      } else {
        isCombo = false;
      }
      if (mounted) {
        setState(() {});
      }
    } else {
      /// 如果为空测量项
      isCombo = false;
      activeChildItemIndex = 0;
      showMore = false;
      activeName = "";
      if (mounted) {
        setState(() {});
      }
    }
  }

  /// 子测量项变化事件监听
  void _onWorkingChildChanged(sender, int e) {
    setState(() {
      activeChildItemIndex = e;
    });
  }

  /// 打开更多测量项选择器
  Future<void> _openMoreMeasureItemDialog() async {
    List<MobileMoreMeasureItemModesModel> modes = [];
    if (_isInitAllMeasureItems) {
      modes = _availableModes;
    } else {
      modes = await _initMeasureItemsList();
      _availableModes = modes;
    }
    final List<String> _currModes =
        _findModesByCurrArea(application.currentVisualArea);

    final List<MobileMoreMeasureItemModesModel> activeModes = modes
        .where((element) => _currModes.contains(element.modeName))
        .toList();

    final MoreMeasureBackResult? result =
        await Get.dialog<MoreMeasureBackResult>(MobileMoreMeasureItemDialog(
      measureModeList: activeModes,
      activeItemName: activeName,
      initModeIndex: showMore ? moreMeasureBackResult?.currModeIndex : null,
      initScrollOffset:
          showMore ? moreMeasureBackResult?.currScrollOffset : null,
    ));
    if (result != null) {
      moreMeasureBackResult = result;
      bool success = changeItem(result.selectedMeasureItemMeta);
      if (success) {
        setState(() {
          showMore = true;
        });
      } else {
        PromptBox.toast(i18nBook.measure.temporarilyUnsupportedMeasurement.t);
      }
    }
  }

  void _visualAreaChanged(sender, IVisualArea e) {
    _cancelCurrSelect();
    _currentAreaType = e.visualAreaType;
  }
}
